"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isEventHandler = exports.isAttributeShorthand = exports.isHTMLElement = exports.isAttributeName = void 0;
/**
 * Returns when given node represents an HTML Attribute.
 * Example: The `class` in `<div class=".."`.
 * Note: This method returns `false` for shorthands like `<div {foo}`.
 */
function isAttributeName(node, only) {
    var _a;
    return !!node && node.type === 'Attribute' && (!only || ((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type) === only);
}
exports.isAttributeName = isAttributeName;
/**
 * Returns when given node represents an HTML element.
 * Example: The `div` in `<div class=".."`.
 * Note: This method returns `false` for shorthands like `<div {foo}`.
 */
function isHTMLElement(node, only) {
    var _a;
    return !!node && node.type === 'Attribute' && (!only || ((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type) === only);
}
exports.isHTMLElement = isHTMLElement;
/**
 * Returns when given node represents an HTML Attribute shorthand or is inside one.
 * Example: The `{foo}` in `<div {foo}`
 */
function isAttributeShorthand(node, only) {
    if (!node) {
        return false;
    }
    do {
        // We could get the expression, or the shorthand, or the attribute
        // Be pragmatic and just go upwards until we can't anymore
        if (isAttributeName(node, only)) {
            console.log('yay');
            return true;
        }
        node = node.parent;
    } while (node);
    return false;
}
exports.isAttributeShorthand = isAttributeShorthand;
/**
 * Returns when given node represents an HTML Attribute shorthand or is inside one.
 * Example: The `on:click={foo}` in `<div on:click={foo}`
 */
function isEventHandler(node, only) {
    var _a;
    return !!node && node.type === 'EventHandler' && (!only || ((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type) === only);
}
exports.isEventHandler = isEventHandler;
//# sourceMappingURL=svelte-ast-utils.js.map