"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSvelteSys = void 0;
const typescript_1 = __importDefault(require("typescript"));
const utils_1 = require("./utils");
/**
 * This should only be accessed by TS svelte module resolution.
 */
function createSvelteSys(getSnapshot) {
    const fileExistsCache = new Map();
    const svelteSys = {
        ...typescript_1.default.sys,
        fileExists(path) {
            var _a;
            path = (0, utils_1.ensureRealSvelteFilePath)(path);
            const exists = (_a = fileExistsCache.get(path)) !== null && _a !== void 0 ? _a : typescript_1.default.sys.fileExists(path);
            fileExistsCache.set(path, exists);
            return exists;
        },
        readFile(path) {
            const snapshot = getSnapshot(path);
            return snapshot.getText(0, snapshot.getLength());
        },
        readDirectory(path, extensions, exclude, include, depth) {
            const extensionsWithSvelte = (extensions !== null && extensions !== void 0 ? extensions : []).concat('.svelte');
            return typescript_1.default.sys.readDirectory(path, extensionsWithSvelte, exclude, include, depth);
        },
        deleteFile(path) {
            var _a, _b;
            fileExistsCache.delete((0, utils_1.ensureRealSvelteFilePath)(path));
            return (_b = (_a = typescript_1.default.sys).deleteFile) === null || _b === void 0 ? void 0 : _b.call(_a, path);
        },
        deleteFromCache(path) {
            fileExistsCache.delete((0, utils_1.ensureRealSvelteFilePath)(path));
        }
    };
    if (typescript_1.default.sys.realpath) {
        const realpath = typescript_1.default.sys.realpath;
        svelteSys.realpath = function (path) {
            if ((0, utils_1.isVirtualSvelteFilePath)(path)) {
                return realpath((0, utils_1.toRealSvelteFilePath)(path)) + '.ts';
            }
            return realpath(path);
        };
    }
    return svelteSys;
}
exports.createSvelteSys = createSvelteSys;
//# sourceMappingURL=svelte-sys.js.map